@@ -20,8 +20,10 @@ package org.apache.activemq.artemis.core.persistence.impl.journal;
 import java.io.File;
 import java.nio.ByteBuffer;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.EnumSet;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -64,6 +66,7 @@ import org.apache.activemq.artemis.core.server.JournalType;
 import org.apache.activemq.artemis.core.server.LargeServerMessage;
 import org.apache.activemq.artemis.core.server.files.FileStoreMonitor;
 import org.apache.activemq.artemis.journal.ActiveMQJournalBundle;
+import org.apache.activemq.artemis.spi.core.protocol.RemotingConnection;
 import org.apache.activemq.artemis.utils.ArtemisCloseable;
 import org.apache.activemq.artemis.utils.ExecutorFactory;
 import org.apache.activemq.artemis.utils.critical.CriticalAnalyzer;
@@ -119,6 +122,17 @@ public class JournalStorageManager extends AbstractJournalStorageManager {
       super(config, analyzer, executorFactory, null, ioExecutors, criticalErrorListener);
    }
 
+   @Override
+   public Set<RemotingConnection> getUsedConnections() {
+      if (replicator == null) {
+         return Collections.emptySet();
+      } else {
+         HashSet<RemotingConnection> usedConnections = new HashSet<>();
+         usedConnections.add(replicator.getBackupTransportConnection());
+         return usedConnections;
+      }
+   }
+
    @Override
    public SequentialFileFactory getJournalSequentialFileFactory() {
       return journalFF;
@@ -222,9 +236,11 @@ public class JournalStorageManager extends AbstractJournalStorageManager {
    }
 
    protected void createDirectories() {
-      checkAndCreateDir(config.getBindingsLocation(), config.isCreateBindingsDir());
-      checkAndCreateDir(config.getJournalLocation(), config.isCreateJournalDir());
-      checkAndCreateDir(config.getLargeMessagesLocation(), config.isCreateJournalDir());
+      if (!config.isUsingDatabasePersistence()) {
+         checkAndCreateDir(config.getBindingsLocation(), config.isCreateBindingsDir());
+         checkAndCreateDir(config.getJournalLocation(), config.isCreateJournalDir());
+         checkAndCreateDir(config.getLargeMessagesLocation(), config.isCreateJournalDir());
+      }
    }
 
    @Override
@@ -271,9 +287,9 @@ public class JournalStorageManager extends AbstractJournalStorageManager {
 
       if (!ioCriticalError) {
          performCachedLargeMessageDeletes();
-         // Must call close to make sure last id is persisted
+         // Must call stop to make sure last id is persisted
          if (journalLoaded && idGenerator != null)
-            idGenerator.persistCurrentID();
+            idGenerator.stop();
       }
 
       final CountDownLatch latch = new CountDownLatch(1);
@@ -637,16 +653,11 @@ public class JournalStorageManager extends AbstractJournalStorageManager {
                originalBindingsJournal.replicationSyncPreserveOldFiles();
                originalMessageJournal.replicationSyncPreserveOldFiles();
 
-               pagingManager.lock();
-               try {
-                  pagingManager.disableCleanup();
-                  messageFiles = prepareJournalForCopy(originalMessageJournal, JournalContent.MESSAGES, nodeID, autoFailBack);
-                  bindingsFiles = prepareJournalForCopy(originalBindingsJournal, JournalContent.BINDINGS, nodeID, autoFailBack);
-                  pageFilesToSync = getPageInformationForSync(pagingManager);
-                  pendingLargeMessages = recoverPendingLargeMessages();
-               } finally {
-                  pagingManager.unlock();
-               }
+               pagingManager.disableCleanup();
+               messageFiles = prepareJournalForCopy(originalMessageJournal, JournalContent.MESSAGES, nodeID, autoFailBack);
+               bindingsFiles = prepareJournalForCopy(originalBindingsJournal, JournalContent.BINDINGS, nodeID, autoFailBack);
+               pageFilesToSync = getPageInformationForSync(pagingManager);
+               pendingLargeMessages = recoverPendingLargeMessages();
             } finally {
                originalMessageJournal.synchronizationUnlock();
                originalBindingsJournal.synchronizationUnlock();
