@@ -36,6 +36,7 @@ import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.Executor;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.ScheduledFuture;
+import java.util.concurrent.ThreadFactory;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 
@@ -62,9 +63,11 @@ import io.netty.channel.local.LocalAddress;
 import io.netty.channel.local.LocalServerChannel;
 import io.netty.channel.nio.NioEventLoopGroup;
 import io.netty.channel.socket.nio.NioServerSocketChannel;
+import io.netty.handler.codec.haproxy.HAProxyMessageDecoder;
 import io.netty.handler.ssl.SslContext;
 import io.netty.handler.ssl.SslHandler;
 import io.netty.util.ResourceLeakDetector;
+import io.netty.util.concurrent.Future;
 import io.netty.util.concurrent.GenericFutureListener;
 import io.netty.util.concurrent.GlobalEventExecutor;
 import org.apache.activemq.artemis.api.config.ActiveMQDefaultConfiguration;
@@ -73,6 +76,7 @@ import org.apache.activemq.artemis.api.core.Pair;
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.api.core.TransportConfiguration;
 import org.apache.activemq.artemis.api.core.management.CoreNotificationType;
+import org.apache.activemq.artemis.api.core.management.ResourceNames;
 import org.apache.activemq.artemis.core.client.impl.ClientSessionFactoryImpl;
 import org.apache.activemq.artemis.core.protocol.ProtocolHandler;
 import org.apache.activemq.artemis.core.remoting.impl.AbstractAcceptor;
@@ -84,6 +88,7 @@ import org.apache.activemq.artemis.core.server.ActiveMQServerLogger;
 import org.apache.activemq.artemis.core.server.cluster.ClusterConnection;
 import org.apache.activemq.artemis.core.server.management.Notification;
 import org.apache.activemq.artemis.core.server.management.NotificationService;
+import org.apache.activemq.artemis.core.server.metrics.MetricsManager;
 import org.apache.activemq.artemis.spi.core.protocol.ProtocolManager;
 import org.apache.activemq.artemis.spi.core.remoting.BufferHandler;
 import org.apache.activemq.artemis.spi.core.remoting.Connection;
@@ -94,6 +99,7 @@ import org.apache.activemq.artemis.spi.core.remoting.ssl.SSLContextConfig;
 import org.apache.activemq.artemis.spi.core.remoting.ssl.SSLContextFactoryProvider;
 import org.apache.activemq.artemis.utils.ActiveMQThreadFactory;
 import org.apache.activemq.artemis.utils.ConfigurationHelper;
+import org.apache.activemq.artemis.utils.ProxyProtocolUtil;
 import org.apache.activemq.artemis.utils.collections.TypedProperties;
 import org.apache.commons.lang3.exception.ExceptionUtils;
 import org.slf4j.Logger;
@@ -141,6 +147,8 @@ public class NettyAcceptor extends AbstractAcceptor {
 
    private final boolean sslEnabled;
 
+   private final boolean proxyProtocolEnabled;
+
    private final boolean useInvm;
 
    private final boolean useEpoll;
@@ -214,10 +222,14 @@ public class NettyAcceptor extends AbstractAcceptor {
 
    private NotificationService notificationService;
 
-   /** The amount of time we wait before new tasks are added during a shutdown period. */
+   /**
+    * The amount of time we wait before new tasks are added during a shutdown period.
+    */
    private int quietPeriod;
 
-   /** The total amount of time we wait before a hard shutdown. */
+   /**
+    * The total amount of time we wait before a hard shutdown.
+    */
    private int shutdownTimeout;
 
    private boolean paused;
@@ -246,6 +258,10 @@ public class NettyAcceptor extends AbstractAcceptor {
 
    private volatile int actualPort = 0;
 
+   private final String threadFactoryGroupName;
+
+   private final MetricsManager metricsManager;
+
    public NettyAcceptor(final String name,
                         final ClusterConnection clusterConnection,
                         final Map<String, Object> configuration,
@@ -253,7 +269,9 @@ public class NettyAcceptor extends AbstractAcceptor {
                         final ServerConnectionLifeCycleListener listener,
                         final ScheduledExecutorService scheduledThreadPool,
                         final Executor failureExecutor,
-                        final Map<String, ProtocolManager> protocolMap) {
+                        final Map<String, ProtocolManager> protocolMap,
+                        final String threadFactoryGroupName,
+                        final MetricsManager metricsManager) {
       super(protocolMap);
 
       this.failureExecutor = failureExecutor;
@@ -268,9 +286,15 @@ public class NettyAcceptor extends AbstractAcceptor {
 
       this.listener = listener;
 
+      this.threadFactoryGroupName = threadFactoryGroupName;
+
+      this.metricsManager = metricsManager;
+
       sslEnabled = ConfigurationHelper.getBooleanProperty(TransportConstants.SSL_ENABLED_PROP_NAME, TransportConstants.DEFAULT_SSL_ENABLED, configuration);
 
-      remotingThreads = ConfigurationHelper.getIntProperty(TransportConstants.NIO_REMOTING_THREADS_PROPNAME, -1, configuration);
+      proxyProtocolEnabled = ConfigurationHelper.getBooleanProperty(TransportConstants.PROXY_PROTOCOL_ENABLED_PROP_NAME, TransportConstants.DEFAULT_PROXY_PROTOCOL_ENABLED, configuration);
+
+      remotingThreads = ConfigurationHelper.getIntProperty(TransportConstants.NIO_REMOTING_THREADS_PROPNAME, Runtime.getRuntime().availableProcessors() * 3, configuration);
       remotingThreads = ConfigurationHelper.getIntProperty(TransportConstants.REMOTING_THREADS_PROPNAME, remotingThreads, configuration);
 
       useEpoll = ConfigurationHelper.getBooleanProperty(TransportConstants.USE_EPOLL_PROP_NAME, TransportConstants.DEFAULT_USE_EPOLL, configuration);
@@ -423,55 +447,40 @@ public class NettyAcceptor extends AbstractAcceptor {
          channelClazz = LocalServerChannel.class;
          eventLoopGroup = new DefaultEventLoopGroup();
       } else {
-
-         if (remotingThreads == -1) {
-            // Default to number of cores * 3
-            remotingThreads = Runtime.getRuntime().availableProcessors() * 3;
-         }
-
+         ThreadFactory threadFactory = AccessController.doPrivileged((PrivilegedAction<ActiveMQThreadFactory>) () -> new ActiveMQThreadFactory(threadFactoryGroupName, true, ClientSessionFactoryImpl.class.getClassLoader()));
          if (useEpoll && CheckDependencies.isEpollAvailable()) {
             channelClazz = EpollServerSocketChannel.class;
-            eventLoopGroup = new EpollEventLoopGroup(remotingThreads, AccessController.doPrivileged(new PrivilegedAction<ActiveMQThreadFactory>() {
-               @Override
-               public ActiveMQThreadFactory run() {
-                  return new ActiveMQThreadFactory("activemq-netty-threads", true, ClientSessionFactoryImpl.class.getClassLoader());
-               }
-            }));
+            eventLoopGroup = new EpollEventLoopGroup(remotingThreads, threadFactory);
             acceptorType = EPOLL_ACCEPTOR_TYPE;
-
-            logger.debug("Acceptor using native epoll");
+            logger.debug("Acceptor {} using native epoll", name);
          } else if (useKQueue && CheckDependencies.isKQueueAvailable()) {
             channelClazz = KQueueServerSocketChannel.class;
-            eventLoopGroup = new KQueueEventLoopGroup(remotingThreads, AccessController.doPrivileged(new PrivilegedAction<ActiveMQThreadFactory>() {
-               @Override
-               public ActiveMQThreadFactory run() {
-                  return new ActiveMQThreadFactory("activemq-netty-threads", true, ClientSessionFactoryImpl.class.getClassLoader());
-               }
-            }));
+            eventLoopGroup = new KQueueEventLoopGroup(remotingThreads, threadFactory);
             acceptorType = KQUEUE_ACCEPTOR_TYPE;
-
-            logger.debug("Acceptor using native kqueue");
+            logger.debug("Acceptor {} using native kqueue", name);
          } else {
             channelClazz = NioServerSocketChannel.class;
-            eventLoopGroup = new NioEventLoopGroup(remotingThreads, AccessController.doPrivileged(new PrivilegedAction<ActiveMQThreadFactory>() {
-               @Override
-               public ActiveMQThreadFactory run() {
-                  return new ActiveMQThreadFactory("activemq-netty-threads", true, ClientSessionFactoryImpl.class.getClassLoader());
-               }
-            }));
+            eventLoopGroup = new NioEventLoopGroup(remotingThreads, threadFactory);
             acceptorType = NIO_ACCEPTOR_TYPE;
-            logger.debug("Acceptor using nio");
+            logger.debug("Acceptor {} using nio", name);
          }
       }
 
+      if (metricsManager != null) {
+         metricsManager.registerNettyEventLoopGroup(name, eventLoopGroup);
+      }
+
       bootstrap = new ServerBootstrap();
       bootstrap.group(eventLoopGroup);
       bootstrap.channel(channelClazz);
 
-      ChannelInitializer<Channel> factory = new ChannelInitializer<Channel>() {
+      ChannelInitializer<Channel> factory = new ChannelInitializer<>() {
          @Override
          public void initChannel(Channel channel) throws Exception {
             ChannelPipeline pipeline = channel.pipeline();
+            if (proxyProtocolEnabled) {
+               pipeline.addLast(new HAProxyMessageEnforcer(getName()), new HAProxyMessageDecoder(), new HAProxyMessageHandler());
+            }
             if (sslEnabled) {
                final Pair<String, Integer> peerInfo = getPeerInfo(channel);
                try {
@@ -480,7 +489,7 @@ public class NettyAcceptor extends AbstractAcceptor {
                   pipeline.addLast("sslHandshakeExceptionHandler", new SslHandshakeExceptionHandler());
                } catch (Exception e) {
                   Throwable rootCause = ExceptionUtils.getRootCause(e);
-                  ActiveMQServerLogger.LOGGER.gettingSslHandlerFailed(channel.remoteAddress().toString(), rootCause.getClass().getName() + ": " + rootCause.getMessage());
+                  ActiveMQServerLogger.LOGGER.gettingSslHandlerFailed(ProxyProtocolUtil.getRemoteAddress(channel), rootCause.getClass().getName() + ": " + rootCause.getMessage());
 
                   logger.debug("Getting SSL handler failed", e);
                   throw e;
@@ -491,7 +500,7 @@ public class NettyAcceptor extends AbstractAcceptor {
 
          private Pair<String, Integer> getPeerInfo(Channel channel) {
             try {
-               String[] peerInfo = channel.remoteAddress().toString().replace("/", "").split(":");
+               String[] peerInfo = ProxyProtocolUtil.getRemoteAddress(channel).replace("/", "").split(":");
                return new Pair<>(peerInfo[0], Integer.parseInt(peerInfo[1]));
             } catch (Exception e) {
                logger.debug("Failed to parse peer info for SSL engine initialization", e);
@@ -580,8 +589,8 @@ public class NettyAcceptor extends AbstractAcceptor {
    }
 
    /**
-    * Transfers the Netty channel that has been created outside of this NettyAcceptor
-    * to control it and configure it according to this NettyAcceptor setting.
+    * Transfers the Netty channel that has been created outside of this NettyAcceptor to control it and configure it
+    * according to this NettyAcceptor setting.
     *
     * @param channel A Netty channel created outside this NettyAcceptor.
     */
@@ -746,10 +755,11 @@ public class NettyAcceptor extends AbstractAcceptor {
    @Override
    public void stop() throws Exception {
       CountDownLatch latch = new CountDownLatch(1);
-
       asyncStop(latch::countDown);
-
       latch.await();
+      if (metricsManager != null) {
+         metricsManager.remove(ResourceNames.ACCEPTOR + this.name);
+      }
    }
 
    @Override
@@ -758,47 +768,63 @@ public class NettyAcceptor extends AbstractAcceptor {
          callback.run();
          return;
       }
+      try {
 
-      if (protocolHandler != null) {
-         protocolHandler.close();
-      }
+         if (protocolHandler != null) {
+            protocolHandler.close();
+         }
 
-      if (batchFlusherFuture != null) {
-         batchFlusherFuture.cancel(false);
+         if (batchFlusherFuture != null) {
+            batchFlusherFuture.cancel(false);
 
-         flusher.cancel();
+            flusher.cancel();
 
-         flusher = null;
+            flusher = null;
 
-         batchFlusherFuture = null;
-      }
+            batchFlusherFuture = null;
+         }
 
-      // serverChannelGroup has been unbound in pause()
-      if (serverChannelGroup != null) {
-         serverChannelGroup.close().awaitUninterruptibly();
-      }
+         // serverChannelGroup has been unbound in pause()
+         if (serverChannelGroup != null) {
+            serverChannelGroup.close().awaitUninterruptibly();
+         }
 
-      if (channelGroup != null) {
-         ChannelGroupFuture future = channelGroup.close().awaitUninterruptibly();
+         if (channelGroup != null) {
+            ChannelGroupFuture future = channelGroup.close().awaitUninterruptibly();
 
-         if (!future.isSuccess()) {
-            ActiveMQServerLogger.LOGGER.nettyChannelGroupError();
-            for (Channel channel : future.group()) {
-               if (channel.isActive()) {
-                  ActiveMQServerLogger.LOGGER.nettyChannelStillOpen(channel, channel.remoteAddress());
+            if (!future.isSuccess()) {
+               ActiveMQServerLogger.LOGGER.nettyChannelGroupError();
+               for (Channel channel : future.group()) {
+                  if (channel.isActive()) {
+                     ActiveMQServerLogger.LOGGER.nettyChannelStillOpen(channel, ProxyProtocolUtil.getRemoteAddress(channel));
+                  }
                }
             }
          }
+
+         channelClazz = null;
+
+         for (Connection connection : connections.values()) {
+            listener.connectionDestroyed(connection.getID(), true);
+         }
+
+         connections.clear();
+
+         paused = false;
+      } finally {
+         if (eventLoopGroup == null) {
+            callback.run();
+         } else {
+            // Shutdown the EventLoopGroup if no new task was added for 100ms or if
+            // 3000ms elapsed.
+            eventLoopGroup.shutdownGracefully(quietPeriod, shutdownTimeout, TimeUnit.MILLISECONDS).addListener(f -> callback.run());
+            eventLoopGroup = null;
+         }
       }
+   }
 
-      channelClazz = null;
-
-      for (Connection connection : connections.values()) {
-         listener.connectionDestroyed(connection.getID(), true);
-      }
-
-      connections.clear();
-
+   @Override
+   public void notifyStop() {
       if (notificationService != null) {
          TypedProperties props = new TypedProperties();
          props.putSimpleStringProperty(SimpleString.of("factory"), SimpleString.of(NettyAcceptorFactory.class.getName()));
@@ -811,13 +837,6 @@ public class NettyAcceptor extends AbstractAcceptor {
             ActiveMQServerLogger.LOGGER.failedToSendNotification(e);
          }
       }
-
-      paused = false;
-
-      // Shutdown the EventLoopGroup if no new task was added for 100ms or if
-      // 3000ms elapsed.
-      eventLoopGroup.shutdownGracefully(quietPeriod, shutdownTimeout, TimeUnit.MILLISECONDS).addListener(f -> callback.run());
-      eventLoopGroup = null;
    }
 
    @Override
@@ -842,7 +861,7 @@ public class NettyAcceptor extends AbstractAcceptor {
             ActiveMQServerLogger.LOGGER.nettyChannelGroupBindError();
             for (Channel channel : future.group()) {
                if (channel.isActive()) {
-                  ActiveMQServerLogger.LOGGER.nettyChannelStillBound(channel, channel.remoteAddress());
+                  ActiveMQServerLogger.LOGGER.nettyChannelStillBound(channel, ProxyProtocolUtil.getRemoteAddress(channel));
                }
             }
          }
@@ -857,8 +876,6 @@ public class NettyAcceptor extends AbstractAcceptor {
 
    /**
     * not allowed
-    *
-    * @param defaultActiveMQPrincipal
     */
    @Override
    public void setDefaultActiveMQPrincipal(ActiveMQPrincipal defaultActiveMQPrincipal) {
@@ -867,8 +884,6 @@ public class NettyAcceptor extends AbstractAcceptor {
 
    /**
     * only InVM acceptors should allow this
-    *
-    * @return
     */
    @Override
    public boolean isUnsecurable() {
@@ -933,14 +948,11 @@ public class NettyAcceptor extends AbstractAcceptor {
 
             SslHandler sslHandler = ctx.pipeline().get(SslHandler.class);
             if (sslHandler != null) {
-               sslHandler.handshakeFuture().addListener(new GenericFutureListener<io.netty.util.concurrent.Future<Channel>>() {
-                  @Override
-                  public void operationComplete(final io.netty.util.concurrent.Future<Channel> future) throws Exception {
-                     if (future.isSuccess()) {
-                        active = true;
-                     } else {
-                        future.getNow().close();
-                     }
+               sslHandler.handshakeFuture().addListener((GenericFutureListener<Future<Channel>>) future -> {
+                  if (future.isSuccess()) {
+                     active = true;
+                  } else {
+                     future.getNow().close();
                   }
                });
             } else {
@@ -948,7 +960,7 @@ public class NettyAcceptor extends AbstractAcceptor {
             }
             return nc;
          } else {
-            ActiveMQServerLogger.LOGGER.connectionLimitReached(connectionsAllowed, ctx.channel().remoteAddress().toString());
+            ActiveMQServerLogger.LOGGER.connectionLimitReached(connectionsAllowed, ProxyProtocolUtil.getRemoteAddress(ctx.channel()));
             ctx.channel().close();
             return null;
          }
@@ -978,13 +990,7 @@ public class NettyAcceptor extends AbstractAcceptor {
       @Override
       public void connectionException(final Object connectionID, final ActiveMQException me) {
          // Execute on different thread to avoid deadlocks
-         new Thread() {
-            @Override
-            public void run() {
-               listener.connectionException(connectionID, me);
-            }
-         }.start();
-
+         new Thread(() -> listener.connectionException(connectionID, me)).start();
       }
 
       @Override
@@ -1039,13 +1045,14 @@ public class NettyAcceptor extends AbstractAcceptor {
             Throwable rootCause = ExceptionUtils.getRootCause(cause);
             String errorMessage = rootCause.getClass().getName() + ": " + rootCause.getMessage();
 
-            ActiveMQServerLogger.LOGGER.sslHandshakeFailed(ctx.channel().remoteAddress().toString(), errorMessage);
+            ActiveMQServerLogger.LOGGER.sslHandshakeFailed(ProxyProtocolUtil.getRemoteAddress(ctx.channel()), errorMessage);
 
             logger.debug("SSL handshake failed", cause);
          }
       }
    }
 
+   @Override
    public boolean isAutoStart() {
       return autoStart;
    }
@@ -1054,4 +1061,8 @@ public class NettyAcceptor extends AbstractAcceptor {
    public int getActualPort() {
       return actualPort;
    }
+
+   public int getRemotingThreads() {
+      return remotingThreads;
+   }
 }
