@@ -40,15 +40,26 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import org.apache.activemq.artemis.core.config.WildcardConfiguration;
 import org.apache.activemq.artemis.core.security.Role;
 import org.apache.activemq.artemis.core.server.ActiveMQServerLogger;
 import org.apache.activemq.artemis.core.server.SecuritySettingPlugin;
 import org.apache.activemq.artemis.core.settings.HierarchicalRepository;
-import org.jboss.logging.Logger;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import java.lang.invoke.MethodHandles;
+import java.util.concurrent.Executors;
+import java.util.concurrent.ScheduledExecutorService;
+import java.util.concurrent.ScheduledFuture;
+import java.util.concurrent.TimeUnit;
+
+import static org.apache.activemq.artemis.core.config.WildcardConfiguration.DEFAULT_WILDCARD_CONFIGURATION;
 
 public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
 
-   private static final Logger logger = Logger.getLogger(LegacyLDAPSecuritySettingPlugin.class);
+   private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
+
+   private static final WildcardConfiguration OPENWIRE_WILDCARD = new WildcardConfiguration().setDelimiter('.').setAnyWords('>').setSingleWord('*');
 
    private static final long serialVersionUID = 4793109879399750045L;
 
@@ -65,8 +76,12 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
    public static final String READ_PERMISSION_VALUE = "readPermissionValue";
    public static final String WRITE_PERMISSION_VALUE = "writePermissionValue";
    public static final String ENABLE_LISTENER = "enableListener";
+   public static final String REFRESH_INTERVAL = "refreshInterval";
    public static final String MAP_ADMIN_TO_MANAGE = "mapAdminToManage";
    public static final String ALLOW_QUEUE_ADMIN_ON_READ = "allowQueueAdminOnRead";
+   public static final String ANY_WORDS_WILDCARD_CONVERSION = "anyWordsWildcardConversion";
+   public static final String SINGLE_WORD_WILDCARD_CONVERSION = "singleWordWildcardConversion";
+   public static final String DELIMITER_WILDCARD_CONVERSION = "delimiterWildcardConversion";
 
    private String initialContextFactory = "com.sun.jndi.ldap.LdapCtxFactory";
    private String connectionURL = "ldap://localhost:1024";
@@ -81,13 +96,20 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
    private String readPermissionValue = "read";
    private String writePermissionValue = "write";
    private boolean enableListener = true;
+   private int refreshInterval = 0;
    private boolean mapAdminToManage = false;
    private boolean allowQueueAdminOnRead = false;
+   private WildcardConfiguration wildcardConfiguration = new WildcardConfiguration()
+      .setAnyWords(DEFAULT_WILDCARD_CONFIGURATION.getAnyWords())
+      .setSingleWord(DEFAULT_WILDCARD_CONFIGURATION.getSingleWord())
+      .setDelimiter(DEFAULT_WILDCARD_CONFIGURATION.getDelimiter());
 
    private DirContext context;
    private EventDirContext eventContext;
    private Map<String, Set<Role>> securityRoles;
    private HierarchicalRepository<Set<Role>> securityRepository;
+   private ScheduledExecutorService scheduler;
+   private ScheduledFuture<?> scheduledFuture;
 
    @Override
    public LegacyLDAPSecuritySettingPlugin init(Map<String, String> options) {
@@ -105,13 +127,40 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
          readPermissionValue = getOption(options, READ_PERMISSION_VALUE, readPermissionValue);
          writePermissionValue = getOption(options, WRITE_PERMISSION_VALUE, writePermissionValue);
          enableListener = getOption(options, ENABLE_LISTENER, Boolean.TRUE.toString()).equalsIgnoreCase(Boolean.TRUE.toString());
+         refreshInterval = Integer.parseInt(getOption(options, REFRESH_INTERVAL, Integer.valueOf(refreshInterval).toString()));
          mapAdminToManage = getOption(options, MAP_ADMIN_TO_MANAGE, Boolean.FALSE.toString()).equalsIgnoreCase(Boolean.TRUE.toString());
          allowQueueAdminOnRead = getOption(options, ALLOW_QUEUE_ADMIN_ON_READ, Boolean.FALSE.toString()).equalsIgnoreCase(Boolean.TRUE.toString());
+         wildcardConfiguration.setAnyWords(getCharOption(options, ANY_WORDS_WILDCARD_CONVERSION, String.valueOf(DEFAULT_WILDCARD_CONFIGURATION.getAnyWords())));
+         wildcardConfiguration.setSingleWord(getCharOption(options, SINGLE_WORD_WILDCARD_CONVERSION, String.valueOf(DEFAULT_WILDCARD_CONFIGURATION.getSingleWord())));
+         wildcardConfiguration.setDelimiter(getCharOption(options, DELIMITER_WILDCARD_CONVERSION, String.valueOf(DEFAULT_WILDCARD_CONFIGURATION.getDelimiter())));
+      }
+
+      if (refreshInterval > 0) {
+         scheduler = Executors.newScheduledThreadPool(1);
+         logger.debug("Scheduling refresh every {} seconds.", refreshInterval);
+         scheduledFuture = scheduler.scheduleAtFixedRate(() -> {
+            logger.debug("Refreshing after {} seconds...", refreshInterval);
+            try {
+               securityRoles = null;
+               securityRepository.swap(getSecurityRoles().entrySet());
+            } catch (Exception e) {
+               ActiveMQServerLogger.LOGGER.unableToRefreshSecuritySettings(e.getMessage());
+               logger.debug("security refresh failure", e);
+            }
+         }, refreshInterval, refreshInterval, TimeUnit.SECONDS);
       }
 
       return this;
    }
 
+   private char getCharOption(Map<String, String> options, String key, String defaultValue) {
+      String result = getOption(options, key, defaultValue);
+      if (result.length() != 1) {
+         throw new IllegalArgumentException("Value for " + key + " is longer than 1 character: " + result);
+      }
+      return result.charAt(0);
+   }
+
    private String getOption(Map<String, String> options, String key, String defaultValue) {
       String result = options.get(key);
       if (result == null) {
@@ -238,6 +287,15 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
       return this;
    }
 
+   public int getRefreshInterval() {
+      return refreshInterval;
+   }
+
+   public LegacyLDAPSecuritySettingPlugin setRefreshInterval(int refreshInterval) {
+      this.refreshInterval = refreshInterval;
+      return this;
+   }
+
    public boolean isMapAdminToManage() {
       return mapAdminToManage;
    }
@@ -256,6 +314,33 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
       return this;
    }
 
+   public char getAnyWordsWildcardConversion() {
+      return this.wildcardConfiguration.getAnyWords();
+   }
+
+   public LegacyLDAPSecuritySettingPlugin setAnyWordsWildcardConversion(char anyWordsWildcardConversion) {
+      this.wildcardConfiguration.setAnyWords(anyWordsWildcardConversion);
+      return this;
+   }
+
+   public char getSingleWordWildcardConversion() {
+      return this.wildcardConfiguration.getSingleWord();
+   }
+
+   public LegacyLDAPSecuritySettingPlugin setSingleWordWildcardConversion(char singleWordWildcardConversion) {
+      this.wildcardConfiguration.setSingleWord(singleWordWildcardConversion);
+      return this;
+   }
+
+   public char getDelimiterWildcardConversion() {
+      return this.wildcardConfiguration.getDelimiter();
+   }
+
+   public LegacyLDAPSecuritySettingPlugin setDelimiterWildcardConversion(char delimiterWildcardConversion) {
+      this.wildcardConfiguration.setDelimiter(delimiterWildcardConversion);
+      return this;
+   }
+
    protected boolean isContextAlive() {
       boolean alive = false;
       if (context != null) {
@@ -328,12 +413,10 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
       securityRoles = new HashMap<>();
       try {
          if (logger.isDebugEnabled()) {
-            logger.debug(new StringBuilder().append("Performing LDAP search: ").append(destinationBase)
-                                            .append("\n\tfilter: ").append(filter)
-                                            .append("\n\tcontrols:")
-                                            .append("\n\t\treturningAttributes: ").append(roleAttribute)
-                                            .append("\n\t\tsearchScope: SUBTREE_SCOPE"));
+            logger.debug("Performing LDAP search: {}\n\tfilter: {}\n\tcontrols:\n\t\treturningAttributes: {}\n\t\tsearchScope: SUBTREE_SCOPE",
+                      destinationBase, filter, roleAttribute);
          }
+
          NamingEnumeration<SearchResult> searchResults = context.search(destinationBase, filter, searchControls);
          while (searchResults.hasMore()) {
             processSearchResult(securityRoles, searchResults.next());
@@ -356,21 +439,24 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
       Attributes attrs = searchResult.getAttributes();
       if (attrs == null || attrs.size() == 0) {
          if (logger.isDebugEnabled()) {
-            logger.debug("Skipping LDAP search result \"" + searchResultLdapName + "\" with " + (attrs == null ? "null" : attrs.size()) + " attributes");
+            logger.debug("Skipping LDAP search result \"{}\" with {} attributes", searchResultLdapName, (attrs == null ? "null" : attrs.size()));
          }
          return;
       }
       List<Rdn> rdns = searchResultLdapName.getRdns();
       if (rdns.size() < 3) {
          if (logger.isDebugEnabled()) {
-            logger.debug("\tSkipping LDAP search result \"" + searchResultLdapName + "\" with " + rdns.size() + " RDNs.");
+            logger.debug("\tSkipping LDAP search result \"{}\" with {} RDNs.", searchResultLdapName, rdns.size());
          }
          return;
       }
-      StringBuilder logMessage = new StringBuilder();
-      if (logger.isDebugEnabled()) {
+
+      final boolean prepareDebugLog = logger.isDebugEnabled();
+      final StringBuilder logMessage = prepareDebugLog ? new StringBuilder() : null;
+      if (prepareDebugLog) {
          logMessage.append("LDAP search result: ").append(searchResultLdapName);
       }
+
       // we can count on the RDNs being in order from right to left
       Rdn rdn = rdns.get(rdns.size() - 3);
       String rawDestinationType = rdn.getValue().toString();
@@ -380,23 +466,23 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
       } else if (rawDestinationType.toLowerCase().contains("topic")) {
          destinationType = "topic";
       }
-      if (logger.isDebugEnabled()) {
+      if (prepareDebugLog) {
          logMessage.append("\n\tDestination type: ").append(destinationType);
       }
 
       rdn = rdns.get(rdns.size() - 2);
-      if (logger.isDebugEnabled()) {
+      if (prepareDebugLog) {
          logMessage.append("\n\tDestination name: ").append(rdn.getValue());
       }
-      String destination = rdn.getValue().toString();
+      String destination = convertDestinationWildcards(rdn.getValue().toString());
 
       rdn = rdns.get(rdns.size() - 1);
-      if (logger.isDebugEnabled()) {
+      if (prepareDebugLog) {
          logMessage.append("\n\tPermission type: ").append(rdn.getValue());
       }
       String permissionType = rdn.getValue().toString();
 
-      if (logger.isDebugEnabled()) {
+      if (prepareDebugLog) {
          logMessage.append("\n\tAttributes: ").append(attrs);
       }
       Attribute attr = attrs.get(roleAttribute);
@@ -414,13 +500,19 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
          LdapName ldapname = new LdapName(value);
          rdn = ldapname.getRdn(ldapname.size() - 1);
          String roleName = rdn.getValue().toString();
-         if (logger.isDebugEnabled()) {
+         if (prepareDebugLog) {
             logMessage.append("\n\tRole name: ").append(roleName);
          }
          boolean write = permissionType.equalsIgnoreCase(writePermissionValue);
          boolean read = permissionType.equalsIgnoreCase(readPermissionValue);
          boolean admin = permissionType.equalsIgnoreCase(adminPermissionValue);
-         Role role = new Role(roleName,
+         Role existingRole = null;
+         for (Role role : roles) {
+            if (role.getName().equals(roleName)) {
+               existingRole = role;
+            }
+         }
+         Role newRole = new Role(roleName,
                               write,                                     // send
                               read,                                      // consume
                               (allowQueueAdminOnRead && read) || admin,  // createDurableQueue
@@ -430,12 +522,18 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
                               mapAdminToManage ? admin : false,          // manage - map to admin based on configuration
                               read,                                      // browse
                               admin,                                     // createAddress
-                              admin);                                    // deleteAddress
-         roles.add(role);
+                              admin,                                     // deleteAddress
+                              read,                                      // view
+                              write);                                    // edit
+         if (existingRole != null) {
+            existingRole.merge(newRole);
+         } else {
+            roles.add(newRole);
+         }
       }
 
-      if (logger.isDebugEnabled()) {
-         logger.debug(logMessage);
+      if (prepareDebugLog) {
+         logger.debug(String.valueOf(logMessage));
       }
 
       if (!exists) {
@@ -443,8 +541,23 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
       }
    }
 
+   private String convertDestinationWildcards(String destination) {
+      // Convert the OpenWire wildcard syntax to the corresponding Artemis syntax
+      destination = OPENWIRE_WILDCARD.convert(destination, wildcardConfiguration);
+      // The '$' character was used previously in LDAP entries to represent the '>' character
+      destination = destination.replace('$', wildcardConfiguration.getAnyWords());
+      return destination;
+   }
+
    @Override
    public SecuritySettingPlugin stop() {
+      if (scheduledFuture != null) {
+         scheduledFuture.cancel(true);
+      }
+      if (scheduler != null) {
+         scheduler.shutdown();
+      }
+
       try {
          eventContext.close();
       } catch (NamingException e) {
@@ -468,16 +581,42 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
     * @param namingEvent the new entry event that occurred
     */
    public void objectAdded(NamingEvent namingEvent) {
-      if (logger.isDebugEnabled()) {
-         logger.debug("objectAdded:\n\told binding: " + namingEvent.getOldBinding() + "\n\tnew binding: " + namingEvent.getNewBinding());
-      }
+      logger.debug("objectAdded:\n\told binding: {}\n\tnew binding: {}", namingEvent.getOldBinding(), namingEvent.getNewBinding());
       Map<String, Set<Role>> newRoles = new HashMap<>();
 
       try {
          processSearchResult(newRoles, (SearchResult) namingEvent.getNewBinding());
-         for (Map.Entry<String, Set<Role>> entry : newRoles.entrySet()) {
-            logger.debug("adding match " + entry.getKey() + ": " + entry.getValue());
-            securityRepository.addMatch(entry.getKey(), entry.getValue());
+         for (Map.Entry<String, Set<Role>> newRole : newRoles.entrySet()) {
+            final String newRoleKey = newRole.getKey();
+            final Set<Role> newRoleValueSet = newRole.getValue();
+
+            Set<Role> existingRoles = securityRepository.getMatch(newRoleKey);
+            if (securityRepository.containsExactWildcardMatch(newRoleKey) || securityRepository.containsExactMatch(newRoleKey) && existingRoles != securityRepository.getDefault()) {
+               Set<Role> merged = new HashSet<>();
+               for (Role existingRole : existingRoles) {
+                  for (Role role : newRoleValueSet) {
+                     if (existingRole.getName().equals(role.getName())) {
+                        if (logger.isDebugEnabled()) {
+                           logger.debug("merging role {} with existing role {} at {}", role, existingRole, newRoleKey);
+                        }
+
+                        existingRole.merge(role);
+                        merged.add(role);
+                     }
+                  }
+               }
+               for (Role role : newRoleValueSet) {
+                  if (!merged.contains(role)) {
+                     logger.debug("add new role {} to existing roles {}", role, existingRoles);
+
+                     existingRoles.add(role);
+                  }
+               }
+            } else {
+               logger.debug("adding new match {}: {}", newRoleKey, newRoleValueSet);
+
+               securityRepository.addMatch(newRoleKey, newRoleValueSet);
+            }
          }
       } catch (NamingException e) {
          ActiveMQServerLogger.LOGGER.failedToProcessEvent(e);
@@ -490,24 +629,20 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
     * @param namingEvent the removed entry event that occurred
     */
    public void objectRemoved(NamingEvent namingEvent) {
-      if (logger.isDebugEnabled()) {
-         logger.debug("objectRemoved:\n\told binding: " + namingEvent.getOldBinding() + "\n\tnew binding: " + namingEvent.getNewBinding());
-      }
+      logger.debug("objectRemoved:\n\told binding: {}\n\tnew binding: {}", namingEvent.getOldBinding(), namingEvent.getNewBinding());
 
       try {
          LdapName ldapName = new LdapName(namingEvent.getOldBinding().getName());
          List<Rdn> rdns = ldapName.getRdns();
          if (rdns.size() < 3) {
             if (logger.isDebugEnabled()) {
-               logger.debug("Skipping old binding name \"" + namingEvent.getOldBinding().getName() + "\" with " + rdns.size() + " RDNs.");
+               logger.debug("Skipping old binding name \"{}\" with {} RDNs.", namingEvent.getOldBinding().getName(), rdns.size());
             }
             return;
          }
 
          String match = rdns.get(rdns.size() - 2).getValue().toString();
-         if (logger.isDebugEnabled()) {
-            logger.debug("Destination name: " + match);
-         }
+         logger.debug("Destination name: {}", match);
 
          if (match != null) {
             Set<Role> roles = securityRepository.getMatch(match);
@@ -516,31 +651,31 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
 
             for (Rdn rdn : ldapName.getRdns()) {
                if (rdn.getValue().equals(writePermissionValue)) {
-                  logger.debug("Removing write permission from " + match);
+                  logger.debug("Removing write permission from {}", match);
                   for (Role role : roles) {
                      if (role.isSend()) {
                         rolesToRemove.add(role);
                      }
                   }
                } else if (rdn.getValue().equals(readPermissionValue)) {
-                  logger.debug("Removing read permission from " + match);
+                  logger.debug("Removing read permission from {}", match);
                   for (Role role : roles) {
                      if (role.isConsume()) {
                         rolesToRemove.add(role);
                      }
                   }
                } else if (rdn.getValue().equals(adminPermissionValue)) {
-                  logger.debug("Removing admin permission from " + match);
+                  logger.debug("Removing admin permission from {}", match);
                   for (Role role : roles) {
                      if (role.isCreateDurableQueue() || role.isCreateNonDurableQueue() || role.isDeleteDurableQueue() || role.isDeleteNonDurableQueue()) {
                         rolesToRemove.add(role);
                      }
                   }
                }
+            }
 
-               for (Role roleToRemove : rolesToRemove) {
-                  roles.remove(roleToRemove);
-               }
+            if (roles.removeAll(rolesToRemove)) {
+               logger.debug("Removed roles: {}. Remaining roles: {}", rolesToRemove, roles);
             }
          }
       } catch (NamingException e) {
@@ -561,9 +696,8 @@ public class LegacyLDAPSecuritySettingPlugin implements SecuritySettingPlugin {
     * @param namingEvent the changed entry event that occurred
     */
    public void objectChanged(NamingEvent namingEvent) {
-      if (logger.isDebugEnabled()) {
-         logger.debug("objectChanged:\n\told binding: " + namingEvent.getOldBinding() + "\n\tnew binding: " + namingEvent.getNewBinding());
-      }
+      logger.debug("objectChanged:\n\told binding: {}\n\tnew binding: {}", namingEvent.getOldBinding(), namingEvent.getNewBinding());
+
       objectRemoved(namingEvent);
       objectAdded(namingEvent);
    }
