@@ -41,10 +41,10 @@ import org.apache.activemq.artemis.api.core.client.ClientMessage;
 import org.apache.activemq.artemis.api.core.client.ClientProducer;
 import org.apache.activemq.artemis.api.core.client.ClientSession;
 import org.apache.activemq.artemis.api.core.client.ClientSessionFactory;
-import org.apache.activemq.artemis.api.core.client.ServerLocator;
 import org.apache.activemq.artemis.core.config.StoreConfiguration;
 import org.apache.activemq.artemis.core.filter.Filter;
 import org.apache.activemq.artemis.core.paging.PagingManager;
+import org.apache.activemq.artemis.core.paging.PagingStore;
 import org.apache.activemq.artemis.core.paging.cursor.PageSubscription;
 import org.apache.activemq.artemis.core.persistence.StorageManager;
 import org.apache.activemq.artemis.core.postoffice.PostOffice;
@@ -53,7 +53,6 @@ import org.apache.activemq.artemis.core.protocol.core.impl.wireformat.SessionCon
 import org.apache.activemq.artemis.core.server.ActiveMQServer;
 import org.apache.activemq.artemis.core.server.MessageReference;
 import org.apache.activemq.artemis.core.server.Queue;
-import org.apache.activemq.artemis.core.server.QueueConfig;
 import org.apache.activemq.artemis.core.server.QueueFactory;
 import org.apache.activemq.artemis.core.server.impl.AckReason;
 import org.apache.activemq.artemis.core.server.impl.ActiveMQServerImpl;
@@ -63,16 +62,22 @@ import org.apache.activemq.artemis.core.settings.HierarchicalRepository;
 import org.apache.activemq.artemis.core.settings.impl.AddressSettings;
 import org.apache.activemq.artemis.jms.client.ActiveMQConnectionFactory;
 import org.apache.activemq.artemis.spi.core.protocol.RemotingConnection;
+import org.apache.activemq.artemis.tests.extensions.parameterized.ParameterizedTestExtension;
 import org.apache.activemq.artemis.tests.integration.largemessage.LargeMessageTestBase;
 import org.apache.activemq.artemis.tests.util.ActiveMQTestBase;
 import org.apache.activemq.artemis.utils.ExecutorFactory;
 import org.apache.activemq.artemis.utils.actors.ArtemisExecutor;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.TestTemplate;
+import org.junit.jupiter.api.extension.ExtendWith;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+
+//Parameters set in superclass
+@ExtendWith(ParameterizedTestExtension.class)
 public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
    private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
@@ -81,25 +86,23 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
    private final int LARGE_MESSAGE_SIZE = ActiveMQClient.DEFAULT_MIN_LARGE_MESSAGE_SIZE * 3;
 
-   protected ServerLocator locator;
-
    public InterruptedLargeMessageTest(StoreConfiguration.StoreType storeType) {
       super(storeType);
    }
 
    @Override
-   @Before
+   @BeforeEach
    public void setUp() throws Exception {
       super.setUp();
       LargeMessageTestInterceptorIgnoreLastPacket.clearInterrupt();
-      locator = createFactory(isNetty());
    }
 
+   @Override
    protected boolean isNetty() {
       return false;
    }
 
-   @Test
+   @TestTemplate
    public void testInterruptLargeMessageSend() throws Exception {
 
       ClientSession session = null;
@@ -117,7 +120,7 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       session = sf.createSession(false, true, true);
 
-      session.createQueue(new QueueConfiguration(ADDRESS));
+      session.createQueue(QueueConfiguration.of(ADDRESS));
 
       ClientProducer producer = session.createProducer(ADDRESS);
 
@@ -135,8 +138,6 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       server.fail(false);
 
-      ActiveMQTestBase.forceGC();
-
       server.start();
 
       server.stop();
@@ -144,7 +145,7 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
       validateNoFilesOnLargeDir();
    }
 
-   @Test
+   @TestTemplate
    public void testCloseConsumerDuringTransmission() throws Exception {
       ActiveMQServer server = createServer(true, isNetty());
 
@@ -158,7 +159,7 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       final ClientSession session = sf.createSession(false, true, true);
 
-      session.createQueue(new QueueConfiguration(ADDRESS));
+      session.createQueue(QueueConfiguration.of(ADDRESS));
 
       ClientProducer producer = session.createProducer(ADDRESS);
 
@@ -176,25 +177,22 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       session.start();
 
-      Thread t = new Thread() {
-         @Override
-         public void run() {
-            try {
-               logger.debug("Receiving message");
-               ClientMessage msg = cons.receive(5000);
-               if (msg == null) {
-                  System.err.println("Message not received");
-                  unexpectedErrors.incrementAndGet();
-                  return;
-               }
-
-               msg.checkCompletion();
-            } catch (ActiveMQException e) {
-               e.printStackTrace();
-               expectedErrors.incrementAndGet();
+      Thread t = new Thread(() -> {
+         try {
+            logger.debug("Receiving message");
+            ClientMessage msg = cons.receive(5000);
+            if (msg == null) {
+               System.err.println("Message not received");
+               unexpectedErrors.incrementAndGet();
+               return;
             }
+
+            msg.checkCompletion();
+         } catch (ActiveMQException e) {
+            e.printStackTrace();
+            expectedErrors.incrementAndGet();
          }
-      };
+      });
 
       t.start();
 
@@ -204,23 +202,23 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       t.join();
 
-      Assert.assertEquals(0, unexpectedErrors.get());
-      Assert.assertEquals(1, expectedErrors.get());
+      assertEquals(0, unexpectedErrors.get());
+      assertEquals(1, expectedErrors.get());
 
       session.close();
 
       server.stop();
    }
 
-   @Test
+   @TestTemplate
    public void testForcedInterruptUsingJMS() throws Exception {
       ActiveMQServer server = createServer(true, isNetty());
 
       server.start();
 
-      SimpleString jmsAddress = new SimpleString("Test");
+      SimpleString jmsAddress = SimpleString.of("Test");
 
-      server.createQueue(new QueueConfiguration(jmsAddress).setRoutingType(RoutingType.ANYCAST));
+      server.createQueue(QueueConfiguration.of(jmsAddress).setRoutingType(RoutingType.ANYCAST));
 
       final AtomicInteger unexpectedErrors = new AtomicInteger(0);
       final AtomicInteger expectedErrors = new AtomicInteger(0);
@@ -230,41 +228,38 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
       connection.start();
       final MessageConsumer consumer = session.createConsumer(session.createQueue(jmsAddress.toString()));
 
-      Thread t = new Thread() {
-         @Override
-         public void run() {
-            try {
-               logger.debug("Receiving message");
-               javax.jms.Message msg = consumer.receive(5000);
-               if (msg == null) {
-                  System.err.println("Message not received");
-                  unexpectedErrors.incrementAndGet();
-                  return;
-               }
-            } catch (JMSException e) {
-               logger.debug("This exception was ok as it was expected", e);
-               expectedErrors.incrementAndGet();
-            } catch (Throwable e) {
-               logger.warn("Captured unexpected exception", e);
+      Thread t = new Thread(() -> {
+         try {
+            logger.debug("Receiving message");
+            javax.jms.Message msg = consumer.receive(5000);
+            if (msg == null) {
+               System.err.println("Message not received");
                unexpectedErrors.incrementAndGet();
+               return;
             }
+         } catch (JMSException e) {
+            logger.debug("This exception was ok as it was expected", e);
+            expectedErrors.incrementAndGet();
+         } catch (Throwable e) {
+            logger.warn("Captured unexpected exception", e);
+            unexpectedErrors.incrementAndGet();
          }
-      };
+      });
 
       t.start();
       t.interrupt();
 
       t.join();
 
-      Assert.assertEquals(0, unexpectedErrors.get());
-      Assert.assertEquals(1, expectedErrors.get());
+      assertEquals(0, unexpectedErrors.get());
+      assertEquals(1, expectedErrors.get());
 
       session.close();
 
       server.stop();
    }
 
-   @Test
+   @TestTemplate
    public void testSendNonPersistentQueue() throws Exception {
 
       ClientSession session = null;
@@ -280,7 +275,7 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       session = sf.createSession(false, true, true);
 
-      session.createQueue(new QueueConfiguration(ADDRESS).setDurable(false));
+      session.createQueue(QueueConfiguration.of(ADDRESS).setDurable(false));
 
       ClientProducer producer = session.createProducer(ADDRESS);
 
@@ -302,9 +297,9 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
       for (int h = 0; h < 5; h++) {
          for (int i = 0; i < 10; i++) {
             ClientMessage clientMessage = cons.receive(5000);
-            Assert.assertNotNull(clientMessage);
+            assertNotNull(clientMessage);
             for (int countByte = 0; countByte < LARGE_MESSAGE_SIZE; countByte++) {
-               Assert.assertEquals(ActiveMQTestBase.getSamplebyte(countByte), clientMessage.getBodyBuffer().readByte());
+               assertEquals(ActiveMQTestBase.getSamplebyte(countByte), clientMessage.getBodyBuffer().readByte());
             }
             clientMessage.acknowledge();
          }
@@ -319,13 +314,13 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
       validateNoFilesOnLargeDir();
    }
 
-   @Test
+   @TestTemplate
    public void testSendPaging() throws Exception {
 
       ClientSession session = null;
 
       LargeMessageTestInterceptorIgnoreLastPacket.disableInterrupt();
-      ActiveMQServer server = createServer(true, createDefaultConfig(isNetty()), 10000, 20000, new HashMap<String, AddressSettings>());
+      ActiveMQServer server = createServer(true, createDefaultConfig(isNetty()), 10000, 20000, new HashMap<>());
 
       // server.getConfiguration()
       // .getIncomingInterceptorClassNames()
@@ -339,7 +334,7 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       session = sf.createSession(false, true, true);
 
-      session.createQueue(new QueueConfiguration(ADDRESS));
+      session.createQueue(QueueConfiguration.of(ADDRESS));
 
       server.getPagingManager().getPageStore(ADDRESS).startPaging();
 
@@ -373,9 +368,9 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
          for (int i = 0; i < 10; i++) {
             ClientMessage clientMessage = cons.receive(5000);
-            Assert.assertNotNull(clientMessage);
+            assertNotNull(clientMessage);
             for (int countByte = 0; countByte < LARGE_MESSAGE_SIZE; countByte++) {
-               Assert.assertEquals(ActiveMQTestBase.getSamplebyte(countByte), clientMessage.getBodyBuffer().readByte());
+               assertEquals(ActiveMQTestBase.getSamplebyte(countByte), clientMessage.getBodyBuffer().readByte());
             }
             clientMessage.acknowledge();
          }
@@ -396,13 +391,13 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
    }
 
-   @Test
+   @TestTemplate
    public void testSendPreparedXA() throws Exception {
       ClientSession session = null;
 
       LargeMessageTestInterceptorIgnoreLastPacket.disableInterrupt();
 
-      ActiveMQServer server = createServer(true, createDefaultConfig(isNetty()), 10000, 20000, new HashMap<String, AddressSettings>());
+      ActiveMQServer server = createServer(true, createDefaultConfig(isNetty()), 10000, 20000, new HashMap<>());
 
       server.getConfiguration().getIncomingInterceptorClassNames().add(LargeMessageTestInterceptorIgnoreLastPacket.class.getName());
 
@@ -417,7 +412,7 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
       Xid xid1 = newXID();
       Xid xid2 = newXID();
 
-      session.createQueue(new QueueConfiguration(ADDRESS));
+      session.createQueue(QueueConfiguration.of(ADDRESS));
 
       ClientProducer producer = session.createProducer(ADDRESS);
 
@@ -467,8 +462,8 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
          for (int i = 0; i < 10; i++) {
             logger.info("I = {}", i);
             ClientMessage msg = cons1.receive(5000);
-            Assert.assertNotNull(msg);
-            Assert.assertEquals(1, msg.getIntProperty("txid").intValue());
+            assertNotNull(msg);
+            assertEquals(1, msg.getIntProperty("txid").intValue());
             msg.acknowledge();
          }
 
@@ -504,28 +499,23 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
       validateNoFilesOnLargeDir();
    }
 
-   @Test
+   @TestTemplate
    public void testRestartBeforeDelete() throws Exception {
 
       class NoPostACKQueue extends QueueImpl {
 
-         NoPostACKQueue(long id,
-                        SimpleString address,
-                        SimpleString name,
-                        Filter filter,
-                        SimpleString user,
-                        PageSubscription pageSubscription,
-                        boolean durable,
-                        boolean temporary,
-                        boolean autoCreated,
-                        ScheduledExecutorService scheduledExecutor,
-                        PostOffice postOffice,
-                        StorageManager storageManager,
-                        HierarchicalRepository<AddressSettings> addressSettingsRepository,
-                        ActiveMQServer server,
-                        ArtemisExecutor executor) {
-            super(id, address, name, filter, pageSubscription != null ? pageSubscription.getPagingStore() : null, pageSubscription, user, durable, temporary, autoCreated, scheduledExecutor,
-                  postOffice, storageManager, addressSettingsRepository, executor, server, null);
+         NoPostACKQueue(QueueConfiguration queueConfiguration,
+                               Filter filter,
+                               PagingStore pagingStore,
+                               PageSubscription pageSubscription,
+                               ScheduledExecutorService scheduledExecutor,
+                               PostOffice postOffice,
+                               StorageManager storageManager,
+                               HierarchicalRepository<AddressSettings> addressSettingsRepository,
+                               ArtemisExecutor executor,
+                               ActiveMQServer server,
+                               QueueFactory factory) {
+            super(queueConfiguration, filter, pagingStore, pageSubscription, scheduledExecutor, postOffice, storageManager, addressSettingsRepository, executor, server, factory);
          }
 
          @Override
@@ -566,38 +556,15 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
             this.server = server;
          }
 
-         @Override
-         public Queue createQueueWith(final QueueConfig config) {
-            return new NoPostACKQueue(config.id(), config.address(), config.name(), config.filter(), config.user(), config.pageSubscription(), config.isDurable(), config.isTemporary(), config.isAutoCreated(), scheduledExecutor, postOffice, storageManager, addressSettingsRepository, server, execFactory.getExecutor());
-         }
-
          @Override
          public Queue createQueueWith(QueueConfiguration config, PagingManager pagingManager, Filter filter) throws Exception {
-            return new NoPostACKQueue(config.getId(), config.getAddress(), config.getName(), filter, config.getUser(), QueueFactoryImpl.getPageSubscription(config, pagingManager, filter), config.isDurable(), config.isTemporary(), config.isAutoCreated(), scheduledExecutor, postOffice, storageManager, addressSettingsRepository, server, execFactory.getExecutor());
+            PageSubscription pageSubscription = QueueFactoryImpl.getPageSubscription(config, pagingManager, filter);
+            return new NoPostACKQueue(config, filter, pageSubscription != null ? pageSubscription.getPagingStore() : null, pageSubscription, scheduledExecutor, postOffice, storageManager, addressSettingsRepository, execFactory.getExecutor(), server, this);
          }
 
-         @Deprecated
-         @Override
-         public Queue createQueue(long persistenceID,
-                                  SimpleString address,
-                                  SimpleString name,
-                                  Filter filter,
-                                  PageSubscription pageSubscription,
-                                  SimpleString user,
-                                  boolean durable,
-                                  boolean temporary,
-                                  boolean autoCreated) {
-
-            return new NoPostACKQueue(persistenceID, address, name, filter, user, pageSubscription, durable, temporary, autoCreated, scheduledExecutor, postOffice, storageManager, addressSettingsRepository, server, execFactory.getExecutor());
-         }
-
-         /* (non-Javadoc)
-          * @see org.apache.activemq.artemis.core.server.QueueFactory#setPostOffice(org.apache.activemq.artemis.core.postoffice.PostOffice)
-          */
          @Override
          public void setPostOffice(PostOffice postOffice) {
          }
-
       }
 
       ClientSession session = null;
@@ -617,7 +584,7 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       session = sf.createSession(false, true, true);
 
-      session.createQueue(new QueueConfiguration(ADDRESS));
+      session.createQueue(QueueConfiguration.of(ADDRESS));
 
       ClientProducer producer = session.createProducer(ADDRESS);
 
@@ -638,7 +605,7 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       for (int i = 0; i < 10; i++) {
          ClientMessage msg = cons.receive(5000);
-         Assert.assertNotNull(msg);
+         assertNotNull(msg);
          msg.saveToOutputStream(new java.io.OutputStream() {
             @Override
             public void write(int b) throws IOException {
@@ -657,7 +624,7 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
       validateNoFilesOnLargeDir();
    }
 
-   @Test
+   @TestTemplate
    public void testConsumeAfterRestart() throws Exception {
       ClientSession session = null;
 
@@ -674,7 +641,7 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       session = sf.createSession(false, true, true);
 
-      session.createQueue(new QueueConfiguration(ADDRESS));
+      session.createQueue(QueueConfiguration.of(ADDRESS));
 
       ClientProducer producer = session.createProducer(ADDRESS);
 
@@ -701,7 +668,7 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       for (int i = 0; i < 10; i++) {
          ClientMessage msg = cons.receive(5000);
-         Assert.assertNotNull(msg);
+         assertNotNull(msg);
          msg.saveToOutputStream(new java.io.OutputStream() {
             @Override
             public void write(int b) throws IOException {
@@ -741,9 +708,8 @@ public class InterruptedLargeMessageTest extends LargeMessageTestBase {
 
       @Override
       public boolean intercept(Packet packet, RemotingConnection connection) throws ActiveMQException {
-         if (packet instanceof SessionContinuationMessage) {
-            SessionContinuationMessage msg = (SessionContinuationMessage) packet;
-            if (!msg.isContinues() && intMessages) {
+         if (packet instanceof SessionContinuationMessage sessionContinuationMessage) {
+            if (!sessionContinuationMessage.isContinues() && intMessages) {
                logger.debug("Ignored a message");
                latch.countDown();
                return false;
