@@ -27,13 +27,15 @@ import javax.jms.ConnectionFactory;
 import javax.jms.DeliveryMode;
 import javax.jms.MessageConsumer;
 import javax.jms.MessageProducer;
+import javax.jms.Queue;
 import javax.jms.Session;
 import javax.jms.TextMessage;
+import javax.jms.Topic;
+import java.io.BufferedOutputStream;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
-import java.util.Arrays;
-import java.util.Collection;
+import java.io.FileOutputStream;
 import java.util.EnumSet;
 import java.util.UUID;
 
@@ -50,6 +52,7 @@ import org.apache.activemq.artemis.api.core.client.ClientSession;
 import org.apache.activemq.artemis.api.core.client.ClientSessionFactory;
 import org.apache.activemq.artemis.api.core.client.ServerLocator;
 import org.apache.activemq.artemis.api.jms.ActiveMQJMSClient;
+import org.apache.activemq.artemis.cli.commands.ActionContext;
 import org.apache.activemq.artemis.cli.commands.tools.xml.XmlDataExporter;
 import org.apache.activemq.artemis.cli.commands.tools.xml.XmlDataImporter;
 import org.apache.activemq.artemis.core.persistence.impl.journal.BatchingIDGenerator;
@@ -57,18 +60,17 @@ import org.apache.activemq.artemis.core.persistence.impl.journal.JournalStorageM
 import org.apache.activemq.artemis.core.persistence.impl.journal.LargeServerMessageImpl;
 import org.apache.activemq.artemis.core.registry.JndiBindingRegistry;
 import org.apache.activemq.artemis.core.server.ActiveMQServer;
+import org.apache.activemq.artemis.core.server.impl.AddressInfo;
 import org.apache.activemq.artemis.core.settings.impl.AddressSettings;
 import org.apache.activemq.artemis.jms.server.JMSServerManager;
 import org.apache.activemq.artemis.jms.server.impl.JMSServerManagerImpl;
-import org.apache.activemq.artemis.tests.extensions.parameterized.ParameterizedTestExtension;
-import org.apache.activemq.artemis.tests.extensions.parameterized.Parameters;
 import org.apache.activemq.artemis.tests.unit.util.InVMContext;
 import org.apache.activemq.artemis.tests.util.ActiveMQTestBase;
-import org.apache.activemq.artemis.tests.util.RandomUtil;
+import org.apache.activemq.artemis.tests.util.CFUtil;
+import org.apache.activemq.artemis.utils.RandomUtil;
 import org.apache.activemq.artemis.tests.util.Wait;
 import org.apache.activemq.artemis.utils.UUIDGenerator;
-import org.junit.jupiter.api.TestTemplate;
-import org.junit.jupiter.api.extension.ExtendWith;
+import org.junit.jupiter.api.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import java.lang.invoke.MethodHandles;
@@ -76,20 +78,11 @@ import java.lang.invoke.MethodHandles;
 /**
  * A test of the XML export/import functionality
  */
-@ExtendWith(ParameterizedTestExtension.class)
 public class XmlImportExportTest extends ActiveMQTestBase {
 
    private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
-   private boolean forceLongs;
-
-   @Parameters(name = "forceLongs={0}")
-   public static Collection getParameters() {
-      return Arrays.asList(new Object[][]{{true}, {false}});
-   }
-
-   public XmlImportExportTest(boolean forceLongs) {
-      this.forceLongs = forceLongs;
+   public XmlImportExportTest() {
 
    }
 
@@ -101,15 +94,15 @@ public class XmlImportExportTest extends ActiveMQTestBase {
    private ClientSessionFactory factory;
    private InVMContext namingContext;
 
-   protected void checkForLongs() {
-      if (forceLongs) {
-         JournalStorageManager manager = (JournalStorageManager) server.getStorageManager();
-         BatchingIDGenerator idGenerator = (BatchingIDGenerator) manager.getIDGenerator();
-         idGenerator.forceNextID((Integer.MAX_VALUE) + 1L);
-      }
+   // this is to force nextID > Integer.MAX_VALUE.
+   // just to make it more challenging if there's any encoding using integer internally by mistake.
+   protected void forceLong() {
+      JournalStorageManager manager = (JournalStorageManager) server.getStorageManager();
+      BatchingIDGenerator idGenerator = (BatchingIDGenerator) manager.getIDGenerator();
+      idGenerator.forceNextID((Integer.MAX_VALUE) + 1L);
    }
 
-   @TestTemplate
+   @Test
    public void testMessageProperties() throws Exception {
       ClientSession session = basicSetUp();
 
@@ -159,7 +152,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       session = factory.createSession(false, true, true);
@@ -177,7 +170,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
          byte[] body = new byte[msg.getBodySize()];
          msg.getBodyBuffer().readBytes(body);
          assertTrue(new String(body).contains("Bob the giant pig " + i));
-         assertEquals(msg.getBooleanProperty("myBooleanProperty"), Boolean.TRUE);
+         assertEquals(Boolean.TRUE, msg.getBooleanProperty("myBooleanProperty"));
          assertEquals(msg.getByteProperty("myByteProperty"), Byte.valueOf("0"));
          byte[] bytes = msg.getBytesProperty("myBytesProperty");
          for (int j = 0; j < 5; j++) {
@@ -204,7 +197,6 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
    /**
     * @return ClientSession
-    * @throws Exception
     */
    private ClientSession basicSetUp() throws Exception {
       server = createServer(true);
@@ -217,11 +209,11 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       jmsServer.start();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
-      checkForLongs();
+      forceLong();
       return addClientSession(factory.createSession(false, true, true));
    }
 
-   @TestTemplate
+   @Test
    public void testMessageTypes() throws Exception {
 
       ClientSession session = basicSetUp();
@@ -258,7 +250,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       session = factory.createSession(false, true, true);
@@ -287,12 +279,9 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       assertEquals(Message.DEFAULT_TYPE, msg.getType());
    }
 
-   @TestTemplate
+   @Test
    public void testTextMessage() throws Exception {
-      StringBuilder data = new StringBuilder();
-      for (int i = 0; i < 2608; i++) {
-         data.append("X");
-      }
+      String data = "X".repeat(2608);
 
       ClientSession session = basicSetUp();
 
@@ -300,7 +289,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       ClientProducer producer = session.createProducer(QUEUE_NAME);
       ClientMessage msg = session.createMessage(Message.TEXT_TYPE, true);
-      msg.getBodyBuffer().writeString(data.toString());
+      msg.getBodyBuffer().writeString(data);
       producer.send(msg);
 
       session.close();
@@ -316,7 +305,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       session = factory.createSession(false, true, true);
@@ -331,15 +320,12 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       msg = consumer.receive(CONSUMER_TIMEOUT);
       assertEquals(Message.TEXT_TYPE, msg.getType());
-      assertEquals(data.toString(), msg.getBodyBuffer().readString());
+      assertEquals(data, msg.getBodyBuffer().readString());
    }
 
-   @TestTemplate
+   @Test
    public void testBytesMessage() throws Exception {
-      StringBuilder data = new StringBuilder();
-      for (int i = 0; i < 2610; i++) {
-         data.append("X");
-      }
+      String data = "X".repeat(2610);
 
       ClientSession session = basicSetUp();
 
@@ -347,7 +333,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       ClientProducer producer = session.createProducer(QUEUE_NAME);
       ClientMessage msg = session.createMessage(Message.BYTES_TYPE, true);
-      msg.getBodyBuffer().writeBytes(data.toString().getBytes());
+      msg.getBodyBuffer().writeBytes(data.getBytes());
       producer.send(msg);
 
       session.close();
@@ -363,7 +349,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       session = factory.createSession(false, true, true);
@@ -380,10 +366,10 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       assertEquals(Message.BYTES_TYPE, msg.getType());
       byte[] result = new byte[msg.getBodySize()];
       msg.getBodyBuffer().readBytes(result);
-      assertEquals(data.toString().getBytes().length, result.length);
+      assertEquals(data.getBytes().length, result.length);
    }
 
-   @TestTemplate
+   @Test
    public void testMessageAttributes() throws Exception {
 
       ClientSession session = basicSetUp();
@@ -412,7 +398,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       session = factory.createSession(false, true, true);
@@ -432,7 +418,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       assertNotNull(msg.getUserID());
    }
 
-   @TestTemplate
+   @Test
    public void testBindingAttributes() throws Exception {
       ClientSession session = basicSetUp();
 
@@ -452,7 +438,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       session = factory.createSession(false, true, true);
@@ -475,7 +461,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       assertTrue(queueQuery.isDurable());
    }
 
-   @TestTemplate
+   @Test
    public void testLargeMessage() throws Exception {
       server = createServer(true);
       server.start();
@@ -483,7 +469,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       factory = createSessionFactory(locator);
       ClientSession session = factory.createSession(false, false);
 
-      LargeServerMessageImpl fileMessage = new LargeServerMessageImpl((JournalStorageManager) server.getStorageManager());
+      LargeServerMessageImpl fileMessage = new LargeServerMessageImpl(server.getStorageManager());
 
       fileMessage.setMessageID(1005);
       fileMessage.setDurable(true);
@@ -519,7 +505,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       session = factory.createSession(false, true, true);
@@ -549,7 +535,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       session.commit();
    }
 
-   @TestTemplate
+   @Test
    public void testLargeMessagesNoTmpFiles() throws Exception {
       server = createServer(true);
       server.start();
@@ -557,7 +543,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       factory = createSessionFactory(locator);
       ClientSession session = factory.createSession(false, false);
 
-      LargeServerMessageImpl fileMessage = new LargeServerMessageImpl((JournalStorageManager) server.getStorageManager());
+      LargeServerMessageImpl fileMessage = new LargeServerMessageImpl(server.getStorageManager());
 
       fileMessage.setMessageID(1005);
       fileMessage.setDurable(true);
@@ -594,7 +580,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       session = factory.createSession(false, true, true);
@@ -637,7 +623,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       }
    }
 
-   @TestTemplate
+   @Test
    public void testLargeJmsTextMessage() throws Exception {
       basicSetUp();
       ConnectionFactory cf = ActiveMQJMSClient.createConnectionFactory("vm://0", "test");
@@ -646,11 +632,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       server.createQueue(QueueConfiguration.of("A").setRoutingType(RoutingType.ANYCAST));
       MessageProducer p = s.createProducer(ActiveMQJMSClient.createQueue("A"));
       p.setDeliveryMode(DeliveryMode.PERSISTENT);
-      StringBuilder stringBuilder = new StringBuilder();
-      for (int i = 0; i < 1024 * 200; i++) {
-         stringBuilder.append(RandomUtil.randomChar());
-      }
-      TextMessage textMessage = s.createTextMessage(stringBuilder.toString());
+      TextMessage textMessage = s.createTextMessage(RandomUtil.randomAlphaNumericString(200 * 1024));
       textMessage.setStringProperty("_AMQ_DUPL_ID", String.valueOf(UUID.randomUUID()));
       p.send(textMessage);
       c.close();
@@ -667,7 +649,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       ClientSession session = factory.createSession(false, true, true);
@@ -690,7 +672,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       c.close();
    }
 
-   @TestTemplate
+   @Test
    public void testPartialQueue() throws Exception {
       ClientSession session = basicSetUp();
 
@@ -722,7 +704,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       session = factory.createSession(false, true, true);
@@ -743,7 +725,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       assertNotNull(msg);
    }
 
-   @TestTemplate
+   @Test
    public void testPagedMessageWithMissingBinding() throws Exception {
       final String MY_ADDRESS = "myAddress";
       final String MY_QUEUE = "myQueue";
@@ -790,7 +772,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = locator.createSessionFactory();
       session = factory.createSession(false, true, true);
@@ -816,7 +798,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       server.stop();
    }
 
-   @TestTemplate
+   @Test
    public void testPaging() throws Exception {
       final String MY_ADDRESS = "myAddress";
       final String MY_QUEUE = "myQueue";
@@ -859,7 +841,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       session = factory.createSession(false, true, true);
@@ -881,7 +863,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       }
    }
 
-   @TestTemplate
+   @Test
    public void testPagedLargeMessage() throws Exception {
       final String MY_ADDRESS = "myAddress";
       final String MY_QUEUE = "myQueue";
@@ -911,7 +893,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
          producer.send(message);
       }
 
-      LargeServerMessageImpl fileMessage = new LargeServerMessageImpl((JournalStorageManager) server.getStorageManager());
+      LargeServerMessageImpl fileMessage = new LargeServerMessageImpl(server.getStorageManager());
 
       fileMessage.setMessageID(1005);
       fileMessage.setDurable(true);
@@ -939,7 +921,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = locator.createSessionFactory();
       session = factory.createSession(false, true, true);
@@ -975,7 +957,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       server.stop();
    }
 
-   @TestTemplate
+   @Test
    public void testTransactional() throws Exception {
       ClientSession session = basicSetUp();
 
@@ -999,7 +981,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = createSessionFactory(locator);
       session = factory.createSession(false, false, true);
@@ -1017,7 +999,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       assertNotNull(msg);
    }
 
-   @TestTemplate
+   @Test
    public void testBody() throws Exception {
       final String QUEUE_NAME = "A1";
       server = createServer(true);
@@ -1047,7 +1029,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = locator.createSessionFactory();
       session = factory.createSession(false, false, true);
@@ -1070,7 +1052,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       server.stop();
    }
 
-   @TestTemplate
+   @Test
    public void testBody2() throws Exception {
       final String QUEUE_NAME = "A1";
       server = createServer(true);
@@ -1105,7 +1087,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = locator.createSessionFactory();
       session = factory.createSession(false, false, true);
@@ -1131,7 +1113,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       server.stop();
    }
 
-   @TestTemplate
+   @Test
    public void testRoutingTypes() throws Exception {
       SimpleString myAddress = SimpleString.of("myAddress");
       ClientSession session = basicSetUp();
@@ -1155,7 +1137,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = locator.createSessionFactory();
       session = factory.createSession(false, false, true);
@@ -1171,7 +1153,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       assertTrue(server.getAddressInfo(myAddress).getRoutingTypes().contains(RoutingType.MULTICAST));
    }
 
-   @TestTemplate
+   @Test
    public void testEmptyRoutingTypes() throws Exception {
       SimpleString myAddress = SimpleString.of("myAddress");
       ClientSession session = basicSetUp();
@@ -1192,7 +1174,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = locator.createSessionFactory();
       session = factory.createSession(false, false, true);
@@ -1207,7 +1189,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       assertEquals(0, server.getAddressInfo(myAddress).getRoutingTypes().size());
    }
 
-   @TestTemplate
+   @Test
    public void testImportWrongRoutingType() throws Exception {
       SimpleString myAddress = SimpleString.of("myAddress");
       SimpleString myQueue = SimpleString.of("myQueue");
@@ -1234,8 +1216,8 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       m.acknowledge();
 
       assertNotNull(m);
-      assertEquals(m.getBodyBuffer().readString(), payload);
-      assertEquals(m.getRoutingType(), RoutingType.ANYCAST);
+      assertEquals(payload, m.getBodyBuffer().readString());
+      assertEquals(RoutingType.ANYCAST, m.getRoutingType());
 
       // Rollback to place ANYCAST message on DLA (MULTICAST)
       session.rollback();
@@ -1256,7 +1238,7 @@ public class XmlImportExportTest extends ActiveMQTestBase {
 
       clearDataRecreateServerDirs();
       server.start();
-      checkForLongs();
+      forceLong();
       locator = createInVMNonHALocator();
       factory = locator.createSessionFactory();
       session = factory.createSession(false, false, true);
@@ -1275,12 +1257,124 @@ public class XmlImportExportTest extends ActiveMQTestBase {
       m = consumer.receive(5000);
 
       assertNotNull(m);
-      assertEquals(m.getBodyBuffer().readString(), payload);
-      assertEquals(m.getRoutingType(), null);
+      assertEquals(payload, m.getBodyBuffer().readString());
+      assertNull(m.getRoutingType());
 
       consumer.close();
       session.close();
       locator.close();
       server.stop();
    }
+
+
+   @Test
+   public void testRemovedQueue() throws Exception {
+
+      String undefinedPrefix = "undef_" + RandomUtil.randomUUIDString() + "_";
+      final int numberOfMessages = 100;
+
+      server = createServer(true, true);
+      server.start();
+      forceLong();
+
+      String anycastQueueName = getTestClassName() + RandomUtil.randomUUIDString();
+      String multicastQueueName = getTestClassName() + RandomUtil.randomUUIDString();
+      createAnycastPair(server, anycastQueueName);
+      server.addAddressInfo(new AddressInfo(multicastQueueName).addRoutingType(RoutingType.MULTICAST).setAutoCreated(false));
+      server.createQueue(QueueConfiguration.of(multicastQueueName).setRoutingType(RoutingType.MULTICAST).setAddress(multicastQueueName));
+
+      org.apache.activemq.artemis.core.server.Queue anycastServerQueue = server.locateQueue(anycastQueueName);
+      assertNotNull(anycastServerQueue);
+      assertEquals(RoutingType.ANYCAST, anycastServerQueue.getRoutingType());
+      org.apache.activemq.artemis.core.server.Queue multiCastServerQueue = server.locateQueue(multicastQueueName);
+      assertNotNull(multiCastServerQueue);
+      assertEquals(RoutingType.MULTICAST, multiCastServerQueue.getRoutingType());
+
+      {
+         ConnectionFactory factory = CFUtil.createConnectionFactory("CORE", "tcp://localhost:61616");
+         try (Connection connection = factory.createConnection()) {
+            Session session = connection.createSession(true, Session.SESSION_TRANSACTED);
+            Topic topic = session.createTopic(multicastQueueName);
+            Queue queue = session.createQueue(anycastQueueName);
+
+            try (MessageProducer producer = session.createProducer(queue)) {
+               for (int i = 0; i < numberOfMessages; i++) {
+                  producer.send(session.createTextMessage("hello " + i));
+               }
+            }
+
+            try (MessageProducer producer = session.createProducer(topic)) {
+               for (int i = 0; i < numberOfMessages; i++) {
+                  producer.send(session.createTextMessage("hello " + i));
+               }
+            }
+
+            session.commit();
+         }
+      }
+
+      // this is forcing a situation where the queue was removed and the messages are still in the journal
+      removeAddressAndQueue(anycastServerQueue);
+      removeAddressAndQueue(multiCastServerQueue);
+
+      server.stop();
+
+      final String fileName = "test.out";
+
+      FileOutputStream fileOutputStream = new FileOutputStream(new File(getTestDir(), fileName));
+      BufferedOutputStream bufferOut = new BufferedOutputStream(fileOutputStream);
+      XmlDataExporter xmlDataExporter = new XmlDataExporter();
+
+      xmlDataExporter.setUndefinedPrefix(undefinedPrefix);
+
+      // the journal should still export even though the bindings don't exist any more
+      // this is to "facilitate" users recovering or undoing mistakes
+      xmlDataExporter.process(bufferOut, getBindingsDir(), getJournalDir(), getPageDir(), getLargeMessagesDir());
+      bufferOut.close();
+      assertNull(xmlDataExporter.getLastError());
+
+      server.start();
+
+      XmlDataImporter importer = new XmlDataImporter();
+      importer.input = new File(getTestDir(), fileName).getAbsolutePath();
+      importer.execute(new ActionContext());
+
+      {
+         ConnectionFactory factory = CFUtil.createConnectionFactory("CORE", "tcp://localhost:61616");
+         try (Connection connection = factory.createConnection()) {
+            Session session = connection.createSession(true, Session.SESSION_TRANSACTED);
+            connection.start();
+
+            Queue anycastJMSQueue = session.createQueue(undefinedPrefix + anycastServerQueue.getID());
+            Queue multicastJMSQueue = session.createQueue(undefinedPrefix + multiCastServerQueue.getID() + "::" + undefinedPrefix + multiCastServerQueue.getID());
+
+            try (MessageConsumer consumer = session.createConsumer(anycastJMSQueue)) {
+               for (int i = 0; i < numberOfMessages; i++) {
+                  TextMessage message = (TextMessage) consumer.receive(5000);
+                  assertNotNull(message);
+                  assertEquals("hello " + i, message.getText());
+               }
+            }
+
+            try (MessageConsumer consumer = session.createConsumer(multicastJMSQueue)) {
+               for (int i = 0; i < numberOfMessages; i++) {
+                  TextMessage message = (TextMessage) consumer.receive(5000);
+                  assertNotNull(message);
+                  assertEquals("hello " + i, message.getText());
+               }
+            }
+
+            session.commit();
+         }
+      }
+   }
+
+   private void removeAddressAndQueue(org.apache.activemq.artemis.core.server.Queue serverQueue) throws Exception {
+      AddressInfo addressInfo = server.getAddressInfo(serverQueue.getAddress());
+      long tx = server.getStorageManager().generateID();
+      server.getStorageManager().deleteAddressBinding(tx, addressInfo.getId());
+      server.getStorageManager().deleteQueueBinding(tx, serverQueue.getID());
+      server.getStorageManager().commitBindings(tx);
+   }
+
 }
