@@ -1,20 +1,21 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements. See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * <br>
- * http://www.apache.org/licenses/LICENSE-2.0
- * <br>
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
-
 package org.apache.activemq.artemis.tests.integration.plugin;
 
 import java.util.Arrays;
@@ -46,6 +47,7 @@ import org.apache.activemq.artemis.core.server.metrics.plugins.SimpleMetricsPlug
 import org.apache.activemq.artemis.core.settings.impl.AddressFullMessagePolicy;
 import org.apache.activemq.artemis.tests.util.ActiveMQTestBase;
 import org.apache.activemq.artemis.tests.util.Wait;
+import org.apache.activemq.artemis.utils.RandomUtil;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 
@@ -62,9 +64,12 @@ public class MetricsPluginTest extends ActiveMQTestBase {
    protected ClientSession session;
    protected ClientSessionFactory sf;
    protected ServerLocator locator;
+   private static final String UUID_NAMESPACE = "uuid";
 
    protected void configureServer(ActiveMQServer server) {
-      server.getConfiguration().setMetricsConfiguration(new MetricsConfiguration().setPlugin(new SimpleMetricsPlugin().init(null)));
+      server.getConfiguration()
+         .setUuidNamespace(UUID_NAMESPACE)
+         .setMetricsConfiguration(new MetricsConfiguration().setPlugin(new SimpleMetricsPlugin().init(null)));
    }
 
    @Override
@@ -87,7 +92,7 @@ public class MetricsPluginTest extends ActiveMQTestBase {
          public final List<Tag> tags;
 
          private Metric(String name, Double value) {
-            this(name, value, Collections.EMPTY_LIST);
+            this(name, value, Collections.emptyList());
          }
 
          private Metric(String name, Double value, List<Tag> tags) {
@@ -102,13 +107,17 @@ public class MetricsPluginTest extends ActiveMQTestBase {
          }
 
          @Override
-         public boolean equals(Object o) {
-            if (this == o) return true;
-            if (o == null || getClass() != o.getClass()) return false;
-            Metric metric = (Metric) o;
-            return Objects.equals(name, metric.name) &&
-               Objects.equals(value, metric.value) &&
-               Objects.equals(tags, metric.tags);
+         public boolean equals(Object obj) {
+            if (this == obj) {
+               return true;
+            }
+            if (!(obj instanceof Metric other)) {
+               return false;
+            }
+
+            return Objects.equals(name, other.name) &&
+                   Objects.equals(value, other.value) &&
+                   Objects.equals(tags, other.tags);
          }
 
          @Override
@@ -136,6 +145,8 @@ public class MetricsPluginTest extends ActiveMQTestBase {
               new Metric("artemis.address.memory.usage.percentage", 0.0, Arrays.asList(Tag.of("broker", "localhost"))),
               new Metric("artemis.connection.count", 1.0, Arrays.asList(Tag.of("broker", "localhost"))),
               new Metric("artemis.total.connection.count", 1.0, Arrays.asList(Tag.of("broker", "localhost"))),
+              new Metric("artemis.session.count", 1.0, Arrays.asList(Tag.of("broker", "localhost"))),
+              new Metric("artemis.total.session.count", 1.0, Arrays.asList(Tag.of("broker", "localhost"))),
               new Metric("artemis.active", 1.0, Arrays.asList(Tag.of("broker", "localhost"))),
               new Metric("artemis.replica.sync", 0.0, Arrays.asList(Tag.of("broker", "localhost"))),
               new Metric("artemis.disk.store.usage", 0.0, Arrays.asList(Tag.of("broker", "localhost"))),
@@ -166,32 +177,44 @@ public class MetricsPluginTest extends ActiveMQTestBase {
               new Metric("artemis.unrouted.message.count", 0.0, Arrays.asList(Tag.of("address", "simpleAddress"), Tag.of("broker", "localhost"))),
               new Metric("artemis.address.size", 0.0, Arrays.asList(Tag.of("address", "simpleAddress"), Tag.of("broker", "localhost"))),
               new Metric("artemis.number.of.pages", 0.0, Arrays.asList(Tag.of("address", "simpleAddress"), Tag.of("broker", "localhost"))),
+              new Metric("artemis.limit.percent", 0.0, Arrays.asList(Tag.of("address", "simpleAddress"), Tag.of("broker", "localhost"))),
               // activemq.notifications metrics
               new Metric("artemis.routed.message.count", 0.0, Arrays.asList(Tag.of("address", "activemq.notifications"), Tag.of("broker", "localhost"))),
               new Metric("artemis.unrouted.message.count", 2.0, Arrays.asList(Tag.of("address", "activemq.notifications"), Tag.of("broker", "localhost"))),
               new Metric("artemis.address.size", 0.0, Arrays.asList(Tag.of("address", "activemq.notifications"), Tag.of("broker", "localhost"))),
-              new Metric("artemis.number.of.pages", 0.0, Arrays.asList(Tag.of("address", "activemq.notifications"), Tag.of("broker", "localhost")))
+              new Metric("artemis.number.of.pages", 0.0, Arrays.asList(Tag.of("address", "activemq.notifications"), Tag.of("broker", "localhost"))),
+              new Metric("artemis.limit.percent", 0.0, Arrays.asList(Tag.of("address", "activemq.notifications"), Tag.of("broker", "localhost")))
       ));
    }
 
    @Test
-   public void testForBasicMetricsPresenceAndValue() throws Exception {
-      internalTestForBasicMetrics(true);
+   public void testForBasicQueueMetricsPresenceAndValue() throws Exception {
+      internalTestForBasicQueueMetrics(true, false);
    }
 
    @Test
-   public void testDisablingMetrics() throws Exception {
-      internalTestForBasicMetrics(false);
+   public void testDisablingQueueMetrics() throws Exception {
+      internalTestForBasicQueueMetrics(false, false);
    }
 
-   private void internalTestForBasicMetrics(boolean enabled) throws Exception {
+   @Test
+   public void testForBasicUuidQueueMetricsPresenceAndValue() throws Exception {
+      internalTestForBasicQueueMetrics(true, true);
+   }
+
+   @Test
+   public void testDisablingUuidQueueMetrics() throws Exception {
+      internalTestForBasicQueueMetrics(false, true);
+   }
+
+   private void internalTestForBasicQueueMetrics(boolean enabled, boolean uuid) throws Exception {
       final String data = "Simple Text " + UUID.randomUUID().toString();
       final String queueName = "simpleQueue";
-      final String addressName = "simpleAddress";
+      final String addressName = uuid ? RandomUtil.randomUUIDString() : "simpleAddress";
 
-      server.getAddressSettingsRepository().getMatch(addressName).setEnableMetrics(enabled);
+      server.getAddressSettingsRepository().getMatch(uuid ? UUID_NAMESPACE + "." + addressName : addressName).setEnableMetrics(enabled);
 
-      session.createQueue(QueueConfiguration.of(queueName).setAddress(addressName).setRoutingType(RoutingType.ANYCAST));
+      session.createQueue(QueueConfiguration.of(queueName).setAddress(addressName).setRoutingType(RoutingType.ANYCAST).setDurable(!uuid));
       ClientProducer producer = session.createProducer(addressName);
       ClientMessage message = session.createMessage(true);
       message.getBodyBuffer().writeString(data);
@@ -206,7 +229,9 @@ public class MetricsPluginTest extends ActiveMQTestBase {
       checkMetric(metrics, "artemis.message.count", "queue", queueName, 1.0, enabled);
       checkMetric(metrics, "artemis.messages.added", "queue", queueName, 1.0, enabled);
       checkMetric(metrics, "artemis.messages.acknowledged", "queue", queueName, 0.0, enabled);
-      checkMetric(metrics, "artemis.durable.message.count", "queue", queueName, 1.0, enabled);
+      if (!uuid) {
+         checkMetric(metrics, "artemis.durable.message.count", "queue", queueName, 1.0, enabled);
+      }
       checkMetric(metrics, "artemis.delivering.message.count", "queue", queueName, 0.0, enabled);
       checkMetric(metrics, "artemis.routed.message.count", "address", addressName, 1.0, enabled);
       checkMetric(metrics, "artemis.unrouted.message.count", "address", addressName, 0.0, enabled);
@@ -284,7 +309,7 @@ public class MetricsPluginTest extends ActiveMQTestBase {
    public static Map<Meter.Id, Double> getMetrics(ActiveMQServer server) {
       Map<Meter.Id, Double> metrics = new HashMap<>();
       List<Meter> meters = server.getMetricsManager().getMeterRegistry().getMeters();
-      assertTrue(meters.size() > 0);
+      assertFalse(meters.isEmpty());
       for (Meter meter : meters) {
          Iterable<Measurement> measurements = meter.measure();
          for (Measurement measurement : measurements) {
